﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Globalization;

using Android.App;
using Android.OS;
using Android.Support.V7.App;
using Android.Runtime;
using Android.Widget;
using Android.Content;

using Newtonsoft.Json;

using Eat.Controls;

namespace Eat.Activities
{
    [Activity(Label = "Products", MainLauncher = true)]
    public class ProductsActivity : AppCompatActivity
    {
        readonly string path;

        EditText addName, addTags, addFat, addCarbs, addProtein, addCost;
        Button addAdd;
        bool productBeignEdited;
        Models.Product productEdited;

        ListView list;
        TextView name, kcal, fat, carbs, protein, cost;
        EditText tags;
        ImageButton search;
        List<Models.Product> products = new List<Models.Product>(), productsFiltered = new List<Models.Product>();
        bool orderedByName, orderedByKcal, orderedByFat, orderedByCarbs, orderedByProtein, orderedByCost;

        public ProductsActivity()
        {
            path = Path.Combine(System.Environment.GetFolderPath(System.Environment.SpecialFolder.Personal), "products.json");

            orderedByName = orderedByKcal = orderedByFat = orderedByCarbs = orderedByProtein = orderedByCost = false;
            productBeignEdited = false;
        }

        protected override void OnCreate(Bundle savedInstanceState)
        {
            base.OnCreate(savedInstanceState);
            SetContentView(Resource.Layout.products);

            DeserializeProducts();
            InitializeControls();
            InitializeAddControls();
        }

        // Save products
        protected override void OnStop()
        {
            base.OnStop();

            SerializeAndSaveProducts();
        }

        void SerializeAndSaveProducts()
        {
            string productsSerialized = JsonConvert.SerializeObject(productsFiltered);
         
            File.WriteAllText(path, productsSerialized);
        }

        void DeserializeProducts()
        {
            try
            {
                string productsSerialized = File.ReadAllText(path);
                products = JsonConvert.DeserializeObject<List<Models.Product>>(productsSerialized);
                productsFiltered = products.ToList();
            }
            catch (Exception e)
            {
                // TODO
                Console.WriteLine(e.Message);

                products = new List<Models.Product>();

                for (int i = 0; i < 3; ++i)
                {
                    products.Add(new Models.Product("Product " + i.ToString(), 1.0f, Convert.ToSingle(i), 1.0f, 1.0f, new List<string>()));
                }
            }
        }

        void InitializeControls()
        {
            productsFiltered = products.ToList();

            list = FindViewById<ListView>(Resource.Id.products_list);
            list.Adapter = new ProductsListViewAdapter(this, productsFiltered);

            list.ItemLongClick += (sender, e) =>
            {
                PopupMenu popup = new PopupMenu(this, e.View);
                popup.Menu.Add(Resource.String.product_edit);
                popup.Menu.Add(Resource.String.product_delete);

                popup.Show();

                popup.MenuItemClick += (_sender, _e) =>
                {
                    // Edit
                    if (_e.Item.TitleFormatted.ToString() == Resources.GetString(Resource.String.product_edit))
                    {
                        productBeignEdited = true;
                        productEdited = productsFiltered[e.Position];

                        addAdd.Text = "Edit";

                        addName.Text = productEdited.Name;
                        addTags.Text = String.Join(" ", productEdited.Tags);
                        addFat.Text = productEdited.Fat.ToString();
                        addCarbs.Text = productEdited.Carbs.ToString();
                        addProtein.Text = productEdited.Protein.ToString();
                        addCost.Text = productEdited.Cost.ToString();
                    }
                    // Remove
                    else if (_e.Item.TitleFormatted.ToString() == Resources.GetString(Resource.String.product_delete))
                    {
                        Models.Product product = productsFiltered[e.Position];
                        productsFiltered.Remove(product);
                        products.Remove(product);

                        list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
                    }
                };
            };

            tags = FindViewById<EditText>(Resource.Id.products_tags);       

            search = FindViewById<ImageButton>(Resource.Id.products_search);
            search.Click += (sender, e) =>
            {
                if (tags.Text == "")
                {
                    productsFiltered = products.ToList();
                }
                else
                {
                    productsFiltered = new List<Models.Product>();
                    List<string> tagsSeparated = tags.Text.Split(' ').ToList();

                    foreach (var tag in tagsSeparated)
                    {
                        productsFiltered.AddRange(products.Where(x => x.Name.ToLower().Contains(tag.ToLower()) || x.Tags.Contains(tag.ToLower())));
                    }
                }

                list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
            };

            name = FindViewById<TextView>(Resource.Id.products_name);
            name.Click += (sender, e) =>
            {
                if (!orderedByName)
                {
                    productsFiltered = productsFiltered.OrderBy(x => x.Name).ToList();
                }
                else
                {
                    productsFiltered = productsFiltered.OrderByDescending(x => x.Name).ToList();
                }
                orderedByName = !orderedByName;
                list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
            };

            kcal = FindViewById<TextView>(Resource.Id.products_kcal);
            kcal.Click += (sender, e) =>
            {
                if (!orderedByKcal)
                {
                    productsFiltered = productsFiltered.OrderBy(x => x.Kcal).ToList();
                }
                else
                {
                    productsFiltered = productsFiltered.OrderByDescending(x => x.Kcal).ToList();
                }
                orderedByKcal = !orderedByKcal;
                list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
            };

            fat = FindViewById<TextView>(Resource.Id.products_fat);
            fat.Click += (sender, e) =>
            {
                if (!orderedByFat)
                {
                    productsFiltered = productsFiltered.OrderBy(x => x.Fat).ToList();
                }
                else
                {
                    productsFiltered = productsFiltered.OrderByDescending(x => x.Fat).ToList();
                }
                orderedByFat = !orderedByFat;
                list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
            };

            carbs = FindViewById<TextView>(Resource.Id.products_carbs);
            carbs.Click += (sender, e) =>
            {
                if (!orderedByCarbs)
                {
                    productsFiltered = productsFiltered.OrderBy(x => x.Carbs).ToList();
                }
                else
                {
                    productsFiltered = productsFiltered.OrderByDescending(x => x.Carbs).ToList();
                }
                orderedByCarbs = !orderedByCarbs;
                list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
            };

            protein = FindViewById<TextView>(Resource.Id.products_protein);
            protein.Click += (sender, e) =>
            {
                if (!orderedByProtein)
                {
                    productsFiltered = productsFiltered.OrderBy(x => x.Protein).ToList();
                }
                else
                {
                    productsFiltered = productsFiltered.OrderByDescending(x => x.Protein).ToList();
                }
                orderedByProtein = !orderedByProtein;
                list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
            };

            cost = FindViewById<TextView>(Resource.Id.products_cost);
            cost.Click += (sender, e) =>
            {
                if (!orderedByCost)
                {
                    productsFiltered = productsFiltered.OrderBy(x => x.Cost).ToList();
                }
                else
                {
                    productsFiltered = productsFiltered.OrderByDescending(x => x.Cost).ToList();
                }
                orderedByCost = !orderedByCost;
                list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
            };
        }

        void InitializeAddControls()
        {
            addName = FindViewById<EditText>(Resource.Id.products_add_name);
            addTags = FindViewById<EditText>(Resource.Id.products_add_tags);
            addFat = FindViewById<EditText>(Resource.Id.products_add_fat);
            addCarbs = FindViewById<EditText>(Resource.Id.products_add_carbs);
            addProtein = FindViewById<EditText>(Resource.Id.products_add_protein);
            addCost = FindViewById<EditText>(Resource.Id.products_add_cost);
            addAdd = FindViewById<Button>(Resource.Id.products_add_add);
           
            addAdd.Click += (sender, e) =>
            {
                if (addName.Text != "" && addFat.Text != "" && addCarbs.Text != "" && addProtein.Text != "" && addCost.Text != "")
                {
                    if (productBeignEdited)
                    {
                        products.Remove(productEdited);
                        productsFiltered.Remove(productEdited);
                        productBeignEdited = false;

                        addAdd.Text = "Add";
                    }

                    Models.Product product;
                    List<string> tags;
                    if (addTags.Text != "")
                    {
                        tags = addTags.Text.Split(' ').ToList();
                    }
                    else
                    {
                        tags = new List<string>();
                    }
                    product = new Models.Product(addName.Text, 
                                                Math.Round(double.Parse(addCost.Text, CultureInfo.InvariantCulture), 2),
                                                Math.Round(double.Parse(addFat.Text, CultureInfo.InvariantCulture), 2),
                                                Math.Round(double.Parse(addCarbs.Text, CultureInfo.InvariantCulture), 2), 
                                                Math.Round(double.Parse(addProtein.Text, CultureInfo.InvariantCulture), 2),
                                                tags);

                    products.Add(product);
                    productsFiltered.Add(product);

                    addName.Text = addTags.Text = addFat.Text = addCarbs.Text = addProtein.Text = addCost.Text = "";

                    list.Adapter = new ProductsListViewAdapter(this, productsFiltered);
                }
            };

        }
    }
}